(()=>{"use strict";var r,e={3749:(r,e,t)=>{var o=t(5311),n=t.n(o);n()((()=>{n()("button[data-action-workflow-status]").on("click",(function(){return ModalWorkflow({url:this.getAttribute("data-url")}),!1}))}))},5311:r=>{r.exports=jQuery}},t={};function o(r){var n=t[r];if(void 0!==n)return n.exports;var a=t[r]={exports:{}};return e[r](a,a.exports,o),a.exports}o.m=e,r=[],o.O=(e,t,n,a)=>{if(!t){var i=1/0;for(s=0;s<r.length;s++){for(var[t,n,a]=r[s],l=!0,u=0;u<t.length;u++)(!1&a||i>=a)&&Object.keys(o.O).every((r=>o.O[r](t[u])))?t.splice(u--,1):(l=!1,a<i&&(i=a));if(l){r.splice(s--,1);var f=n();void 0!==f&&(e=f)}}return e}a=a||0;for(var s=r.length;s>0&&r[s-1][2]>a;s--)r[s]=r[s-1];r[s]=[t,n,a]},o.n=r=>{var e=r&&r.__esModule?()=>r.default:()=>r;return o.d(e,{a:e}),e},o.d=(r,e)=>{for(var t in e)o.o(e,t)&&!o.o(r,t)&&Object.defineProperty(r,t,{enumerable:!0,get:e[t]})},o.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(r){if("object"==typeof window)return window}}(),o.o=(r,e)=>Object.prototype.hasOwnProperty.call(r,e),o.r=r=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(r,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(r,"__esModule",{value:!0})},o.j=711,(()=>{var r={711:0};o.O.j=e=>0===r[e];var e=(e,t)=>{var n,a,[i,l,u]=t,f=0;if(i.some((e=>0!==r[e]))){for(n in l)o.o(l,n)&&(o.m[n]=l[n]);if(u)var s=u(o)}for(e&&e(t);f<i.length;f++)a=i[f],o.o(r,a)&&r[a]&&r[a][0](),r[a]=0;return o.O(s)},t=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];t.forEach(e.bind(null,0)),t.push=e.bind(null,t.push.bind(t))})();var n=o.O(void 0,[751],(()=>o(3749)));n=o.O(n)})();