(()=>{"use strict";var o,t={3152:(o,t,e)=>{var n=e(5311),r=e.n(n);function a(o,t,e){const n=document.createElement("input");n.type="hidden",n.name=t,n.value=e,o.appendChild(n)}window._addHiddenInput=a,window.ActivateWorkflowActionsForDashboard=function(o){document.querySelectorAll("[data-workflow-action-url]").forEach((t=>{t.addEventListener("click",(e=>{if(e.preventDefault(),e.stopPropagation(),"launchModal"in t.dataset)ModalWorkflow({url:t.dataset.workflowActionUrl,onload:{action(o){const t=document.createElement("input");t.type="hidden",t.name="next",t.value=window.location,r()("form",o.body).append(t),o.ajaxifyForm(r()("form",o.body))},success(o,t){window.location.href=t.redirect}}});else{const e=document.createElement("form");e.action=t.dataset.workflowActionUrl,e.method="POST",a(e,"csrfmiddlewaretoken",o),a(e,"next",window.location),document.body.appendChild(e),e.submit()}}),{capture:!0})}))},window.ActivateWorkflowActionsForEditView=function(o){const t=r()(o).get(0);document.querySelectorAll("[data-workflow-action-name]").forEach((o=>{o.addEventListener("click",(e=>{"workflowActionModalUrl"in o.dataset?(e.preventDefault(),e.stopPropagation(),ModalWorkflow({url:o.dataset.workflowActionModalUrl,onload:{action(o){o.ajaxifyForm(r()("form",o.body))},success(e,n){a(t,"action-workflow-action","true"),a(t,"workflow-action-name",o.dataset.workflowActionName),a(t,"workflow-action-extra-data",JSON.stringify(n.cleaned_data)),r()(t).submit()}}})):(a(t,"action-workflow-action","true"),a(t,"workflow-action-name",o.dataset.workflowActionName))}),{capture:!0})}))}},5311:o=>{o.exports=jQuery}},e={};function n(o){var r=e[o];if(void 0!==r)return r.exports;var a=e[o]={exports:{}};return t[o](a,a.exports,n),a.exports}n.m=t,o=[],n.O=(t,e,r,a)=>{if(!e){var i=1/0;for(f=0;f<o.length;f++){for(var[e,r,a]=o[f],l=!0,c=0;c<e.length;c++)(!1&a||i>=a)&&Object.keys(n.O).every((o=>n.O[o](e[c])))?e.splice(c--,1):(l=!1,a<i&&(i=a));if(l){o.splice(f--,1);var d=r();void 0!==d&&(t=d)}}return t}a=a||0;for(var f=o.length;f>0&&o[f-1][2]>a;f--)o[f]=o[f-1];o[f]=[e,r,a]},n.n=o=>{var t=o&&o.__esModule?()=>o.default:()=>o;return n.d(t,{a:t}),t},n.d=(o,t)=>{for(var e in t)n.o(t,e)&&!n.o(o,e)&&Object.defineProperty(o,e,{enumerable:!0,get:t[e]})},n.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(o){if("object"==typeof window)return window}}(),n.o=(o,t)=>Object.prototype.hasOwnProperty.call(o,t),n.r=o=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(o,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(o,"__esModule",{value:!0})},n.j=477,(()=>{var o={477:0};n.O.j=t=>0===o[t];var t=(t,e)=>{var r,a,[i,l,c]=e,d=0;if(i.some((t=>0!==o[t]))){for(r in l)n.o(l,r)&&(n.m[r]=l[r]);if(c)var f=c(n)}for(t&&t(e);d<i.length;d++)a=i[d],n.o(o,a)&&o[a]&&o[a][0](),o[a]=0;return n.O(f)},e=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];e.forEach(t.bind(null,0)),e.push=t.bind(null,e.push.bind(e))})();var r=n.O(void 0,[751],(()=>n(3152)));r=n.O(r)})();